from weather import weatherdata
import scipy.optimize
import scipy.special
import math
import xlwt

def funcK(k, lowBin, highBin, binSize, probDict):
    vi = 0.
    vlogsum = 0.0
    logsum = 0.0
    vsum = 0.0

    for bin in range(lowBin, highBin+1):
        if(bin in probDict):
            vi = bin + binSize / 2.
            vlogsum = vlogsum + (math.pow(vi, k) * math.log(vi) * probDict[bin])
            logsum = logsum + ( math.log(vi) * probDict[bin])
            vsum = vsum + ( math.pow(vi,k) * probDict[bin])

    return (math.pow(vlogsum / vsum - logsum/1.0, -1.0)) - k

def calcK(lowBin, highBin, binSize, probDict):
    return scipy.optimize.fsolve(funcK, 2., args=(lowBin, highBin, binSize, probDict))

def calcMML(year, month, dataDir, binSize):

    print "year: %d, month: %d, dataDir: %s, binSize: %d" % ( year, month, dataDir, binSize)
    data = weatherdata.loadDataCAIR(month, year, dataDir)
    dataLen = len(data) * 1.0
    if(dataLen==0):
        print "No data for %d/%d" % ( month, year)
        return

    zeroCount = 0
    dataDict = {}
    windSpeedSum = 0.0

    for epoch, windDir, windSpeed, temp, pressure in data:
        bin = int(windSpeed / binSize)

        if(not bin in dataDict):
            dataDict[bin]=1
        else:
            dataDict[bin] = dataDict[bin] + 1

        if(windSpeed<=0.01):
            zeroCount = zeroCount + 1
        
        windSpeedSum = windSpeedSum + windSpeed
    
    for bin, count in dataDict.items():
        dataDict[bin] = dataDict[bin]/dataLen


    highBin = max(dataDict.keys())
    lowBin = min(dataDict.keys())
    
    k = calcK(lowBin, highBin, binSize, dataDict)
    
    fd = open('mml-%d-%d.dat' % (month,year), 'a')
    for bin, prob in dataDict.items():
        fd.write('%f\t%f\n' % (bin+binSize/2., prob))

    avgWindSpeed = windSpeedSum / dataLen

    pGreaterZero = 1.0 - zeroCount / dataLen
    
    # equation 3 from mml paper
    c = avgWindSpeed / scipy.special.gamma(1.0 + 1.0/k)
    print "month: %d, year: %d, k: %f, c: %f, avg: %f" % ( month, year, k, c, avgWindSpeed)

    
    return (k, c , pGreaterZero, avgWindSpeed)

def saveFit(fitDict, fitXLSFile):
    wb = xlwt.Workbook()
    ws = wb.add_sheet('K')
    wa = wb.add_sheet('C')
    wz = wb.add_sheet('Prob Speed > 0')
    wv = wb.add_sheet('Average Wind Speed')

    years = fitDict.keys()

    # save the K fit parameters

    ws.write(0,0, 'K')

    months = [ 'jan', 'feb', 'mar', 'apr', 'may', 'june', 'july', 'aug', 'sept', 'oct', 'nov', 'dec'] 
    ws.write(1,0, 'Year')
    for index, m in enumerate(months):
        ws.write(1,index+1, m)

    row = 2
    
    for index, year in enumerate(years):
        ws.write(row+index, 0, year)
        
        for month in range(1,13):
            try:
                a = fitDict[year][month][0]
            except:
                # something wrong getting the fit
                continue

            
            ws.write(row+index, month, fitDict[year][month][0])

    
    # write in an average for all known years
    cols = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N' ]
    row = row+len(years)+1
    for month in range(1,13):
        ws.write(row, month, xlwt.Formula('AVERAGE(%s%d:%s%d)' % (cols[month], 3, cols[month], 2+len(years))))

    # save the C fit parameters
    wa.write(0,0, 'C')
    wa.write(1,0, 'Year')
    for index, m in enumerate(months):
        wa.write(1,index+1, m)

    row = 2
    for index, year in enumerate(years):
        wa.write(row+index, 0, year)
        
        for month in range(1,13):
            try:
                a = fitDict[year][month][1]
            except:
                # skip missing data
                continue

            wa.write(row+index, month, fitDict[year][month][1])

    # write in an average for all known years
    cols = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N' ]
    row = row+len(years)+1
    for month in range(1,13):
        wa.write(row, month, xlwt.Formula('AVERAGE(%s%d:%s%d)' % (cols[month], 3, cols[month], 2+len(years))))


    # save the probability of the wind speed being greater than 0
    wz.write(0,0, 'prob(wind speed)>0')
    wz.write(1,0, 'Year')
    for index, m in enumerate(months):
        wz.write(1,index+1, m)

    row = 2
    for index, year in enumerate(years):
        wz.write(row+index, 0, year)
        
        for month in range(1,13):
            try:
                a = fitDict[year][month][2]
            except:
                # skip missing data
                continue
            wz.write(row+index, month, fitDict[year][month][2])


    # write in an average for all known years
    cols = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N' ]
    row = row+len(years)+1
    for month in range(1,13):
        wz.write(row, month, xlwt.Formula('AVERAGE(%s%d:%s%d)' % (cols[month], 3, cols[month], 2+len(years))))
    

    # save the average wind speed
    wv.write(0,0, 'avg wind speed')
    wv.write(1,0, 'Year')
    for index, m in enumerate(months):
        wv.write(1,index+1, m)

    row = 2
    for index, year in enumerate(years):
        wv.write(row+index, 0, year)
        
        for month in range(1,13):
            try:
                a = fitDict[year][month][3]
            except:
                # skip missing data
                continue

            wv.write(row+index, month, fitDict[year][month][3])


    # write in an average for all known years
    cols = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N' ]
    row = row+len(years)+1
    for month in range(1,13):
        wv.write(row, month, xlwt.Formula('AVERAGE(%s%d:%s%d)' % (cols[month], 3, cols[month], 2+len(years))))

    wb.save(fitXLSFile)

    
if __name__=="__main__":
    fitDict = {}
    for year in [ 2005,2006,2007,2008,2009 ]:
        if(not year in fitDict):
            fitDict[year]={}

        for month in range(1,13):
            fitDict[year][month ] = calcMML(year, month, "/home/mnewcomb/work/mnewcomb/wind/trunk/generators/bergey-xl1/data", 1)

    
    saveFit(fitDict, 'fit.xls')

            
