import glob
import os
import os.path
import time
import re

class weatherdata:
    
    @staticmethod
    def timeStrToEpoch(date, timeStr):
        try:
            strTime = time.strptime("%s %s" % ( date, timeStr), "%m/%d/%y %H:%M:%S")
        except ValueError:
            try:
                strTime = time.strptime("%s %s" % ( date, timeStr), "%m/%d/%Y %H:%M:%S")
            except ValueError, e:
                raise e

        epoch = time.mktime(strTime)

        return epoch

    @staticmethod
    def getYear(filename):

        match = re.match(".*(\d\d)(\d\d)\.txt", filename)
        
        month, year = match.groups()

        year = int(year)

        if(year<20):
            return year+2000
        else:
            return year+1900

    @staticmethod
    def getMonth(filename):

        match = re.match(".*(\d\d)(\d\d)\.txt", filename)
        
        month, year = match.groups()

        month = int(month)

        return month



    @staticmethod
    def loadDataCAIR(month, year, dir):

        data = []

        files = glob.glob(os.path.join(dir, '*.txt'))
        
        for file in files:
            if(weatherdata.getYear(file)!=year):
                #print "not the right year, skipping: %s" % file
                continue

            if(weatherdata.getMonth(file)!=month):
                continue

            print "Processing ---------------------- %s" % file

            fd = open(file,'r')
            for line in fd:
                splitData = line.strip().split()
                
                if(len(splitData)!=13):
                    if(len(splitData)>0 and splitData[0]=="Date"):
                        continue
                    
                    print "Line too short ( expecting 13 got %d ): %s" % ( len(splitData), line)
                   #print splitData
                    continue
        
                try:
                    skipLine=False
                    for idx in [3,4,5,6,10,11,12]:
                        if(splitData[idx]=='M'):
                            # bad sensor value..  skip
                            skipLine=True
                            break
                    if(skipLine):
                        continue

                
                    date = splitData[0]
                    if(date=='UTCDate' or date=='(UTC)' or date=='Date'):
                        continue

                    timeStr = splitData[1]

                    epoch = weatherdata.timeStrToEpoch(date,timeStr)

                    windDir = float(splitData[3])
                    windSpeed = float(splitData[4])
                    
                    temp1 = float(splitData[5])
                    temp2 = float(splitData[6])
                    
                    p1 = float(splitData[10])
                    p2 = float(splitData[11])
                    p3 = float(splitData[12])
                    
                except ValueError,e:
                    print "Bad line: %s" % line
                    print "Exception: " , e
                    continue

            
                dataPt = ( epoch, windDir, windSpeed, min(temp1, temp2), min(p1,p2,p3))
                data.append(dataPt)

                if(windSpeed<0):
                    print "BAD WINDSPEED!"
                    pass

            fd.close()
        
        # sort the data by time
        data.sort(cmp=lambda x,y: cmp(x[0],y[0]))
    
        print "data loaded: %d points" % len(data)
        return data


